import {Component, Inject, OnInit} from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {ConfigService} from "@services/config.service";
import {HttpClient} from "@angular/common/http";
import {ToastrService} from "ngx-toastr";

@Component({
  selector: 'app-software-profile-partition',
  templateUrl: './software-profile-partition.component.html',
  styleUrl: './software-profile-partition.component.css'
})
export class SoftwareProfilePartitionComponent implements OnInit{
  baseUrl: string;
  selectedPartition: any = null;
  dataSource = new MatTableDataSource<any>();
  clientId: string | null = null;
  selectedClients: any[] = [];
  selectedModelClient: any = null;
  filteredPartitions: any[] = [];
  allSelected: boolean = false;
  clientData: any[] = [];
  loading: boolean = false;
  columns = [
    {
      columnDef: 'diskNumber',
      header: 'Disco',
      cell: (partition: any) => partition.diskNumber
    },
    {
      columnDef: 'partitionNumber',
      header: 'Particion',
      cell: (partition: any) => partition.partitionNumber
    },
    {
      columnDef: 'size',
      header: 'Tamaño',
      cell: (partition: any) => `${partition.size} MB`
    },
    {
      columnDef: 'partitionCode',
      header: 'Tipo de partición',
      cell: (partition: any) => partition.partitionCode
    },
    {
      columnDef: 'filesystem',
      header: 'Sistema de ficheros',
      cell: (partition: any) => partition.filesystem
    },
    {
      columnDef: 'operativeSystem',
      header: 'SO',
      cell: (partition: any) => partition.operativeSystem?.name
    }
  ];

  displayedColumns = ['select', ...this.columns.map(column => column.columnDef)];

  constructor(
    @Inject(MAT_DIALOG_DATA) public data: { client: any },
    private dialogRef: MatDialogRef<SoftwareProfilePartitionComponent>,
    private configService: ConfigService,
    private http: HttpClient,
    private toastService: ToastrService,
  ) {
    this.baseUrl = this.configService.apiUrl;
    this.clientId = this.data.client?.clientId
  }

  ngOnInit() {
    this.loadPartitions();
  }

  loadPartitions() {
    const url = `${this.baseUrl}/clients/${this.data.client?.clientId}`;
    this.http.get(url).subscribe(
      (response: any) => {
        if (response.partitions) {
          this.dataSource.data = response.partitions;
        }
      },
      (error) => {
        console.error('Error al cargar los datos del cliente:', error);
      }
    );
  }

  close() {
    this.dialogRef.close();
  }

  execute(): void {
    this.loading = true;

    this.http.post(`${this.baseUrl}/clients/server/${this.data.client.clientId}/software-inventory`, {
      partition: this.selectedPartition['@id'],
    }).subscribe(
      response => {
        this.toastService.success('Inventario de software actualizado correctamente');
        this.dialogRef.close(response);
      },
      error => {
        this.toastService.error(error.error['hydra:description'] || 'Error al actualizar el inventario de software');
      }
    );
  }
}
