import {Component, EventEmitter, OnInit, Output, ViewChild, ElementRef} from '@angular/core';
import { HttpClient } from "@angular/common/http";
import { ToastrService } from "ngx-toastr";
import { ActivatedRoute, Router } from "@angular/router";
import { MatTableDataSource } from "@angular/material/table";
import { SelectionModel } from "@angular/cdk/collections";
import { ConfigService } from '@services/config.service';
import {MatDialog} from "@angular/material/dialog";
import {QueueConfirmationModalComponent} from "../../../../../shared/queue-confirmation-modal/queue-confirmation-modal.component";
import {CreateRepositoryModalComponent} from "./create-repository-modal/create-repository-modal.component";

@Component({
  selector: 'app-create-image',
  templateUrl: './create-image.component.html',
  styleUrl: './create-image.component.css'
})
export class CreateClientImageComponent implements OnInit{
  baseUrl: string;
  @Output() dataChange = new EventEmitter<any>();
  @ViewChild('partitionSection', { static: false }) partitionSection!: ElementRef;

  errorMessage = '';
  clientId: string | null = null;
  partitions: any[] = [];
  images: any[] = [];
  clientName: string = '';
  private _selectedPartition: any = null;
  name: string = '';
  client: any = null;
  loading: boolean = false;
  selectedImage: any = null;
  private _imageType: string = 'monolithic';
  selectedRepository: any = null;
  gitRepositories: any[] = [];
  selectedGitRepository: any = null;
  gitImageRepositories: any[] = [];
  gitImageName: string = '';
  loadingGitRepositories: boolean = false;
  loadingGitImageRepositories: boolean = false;
  creatingRepository: boolean = false;
  gitAction: string = 'create';
  monolithicAction: string = 'create';
  existingImages: any[] = [];
  selectedExistingImage: any = null;
  loadingExistingImages: boolean = false;
  dataSource = new MatTableDataSource<any>();
  columns = [
    {
      columnDef: 'diskNumber',
      header: 'Disco',
      cell: (partition: any) => partition.diskNumber
    },
    {
      columnDef: 'partitionNumber',
      header: 'Particion',
      cell: (partition: any) => partition.partitionNumber
    },
    {
      columnDef: 'size',
      header: 'Tamaño',
      cell: (partition: any) => `${partition.size} MB`
    },
    {
      columnDef: 'partitionCode',
      header: 'Tipo de partición',
      cell: (partition: any) => partition.partitionCode
    },
    {
      columnDef: 'filesystem',
      header: 'Sistema de ficheros',
      cell: (partition: any) => partition.filesystem
    },
    {
      columnDef: 'operativeSystem',
      header: 'SO',
      cell: (partition: any) => partition.operativeSystem?.name
    }
  ];

  displayedColumns = ['select', ...this.columns.map(column => column.columnDef)];
  selection = new SelectionModel(true, []);

  constructor(
    private http: HttpClient,
    private toastService: ToastrService,
    private configService: ConfigService,
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog
  ) {
    this.baseUrl = this.configService.apiUrl;
  }

  ngOnInit() {
    console.log('CreateImageComponent ngOnInit ejecutado');
    this.clientId = this.route.snapshot.paramMap.get('id');
    this.loadPartitions();
    this.loadImages();
  }

  loadPartitions() {
    const url = `${this.baseUrl}/clients/${this.clientId}`;
    this.http.get(url).subscribe(
      (response: any) => {
        if (response.partitions) {
          this.client = response;
          this.clientName = response.name;
          this.selectedRepository = response.repository;

          this.dataSource.data = response.partitions.filter((partition: any) => {
            return partition.partitionNumber !== 0;
          });
        }
      },
      (error) => {
        console.error('Error al cargar los datos del cliente:', error);
      }
    );
  }

  onImageTypeSelected(event: any) {
    this.imageType = event;
  }

  loadImages() {
    const url = `${this.baseUrl}/images?created=false&type=${this.imageType}&page=1&itemsPerPage=100`;
    this.http.get(url).subscribe(
      (response: any) => {
        this.images = response['hydra:member'];
      },
      (error) => {
        console.error('Error al cargar las imágenes:', error);
      }
    );
  }

  loadGitRepositories() {
    this.loadingGitRepositories = true;
    const url = `${this.baseUrl}/git-repositories?repository=${this.selectedRepository.id}&page=1&itemsPerPage=100`;
    return this.http.get(url).subscribe(
      (response: any) => {
        this.gitRepositories = response['hydra:member'];
        this.loadingGitRepositories = false;
      },
      (error) => {
        console.error('Error al cargar los repositorios git:', error);
        this.loadingGitRepositories = false;
      }
    );
  }

  loadGitImageRepositories(gitRepository: any) {
    this.loadingGitImageRepositories = true;
    const url = `${this.baseUrl}/git-image-repositories?gitRepository.id=${gitRepository.id}&page=1&itemsPerPage=100`;
    this.http.get(url).subscribe(
      (response: any) => {
        this.gitImageRepositories = response['hydra:member'];
        this.loadingGitImageRepositories = false;
      },
      (error) => {
        console.error('Error al cargar las imágenes de repositorio git:', error);
        this.loadingGitImageRepositories = false;
      }
    );
  }

  onGitRepositorySelected(gitRepository: any) {
    this.selectedGitRepository = gitRepository;
    this.selectedExistingImage = null;
    this.existingImages = [];
    if (gitRepository) {
      this.loadGitImageRepositories(gitRepository);
    } else {
      this.gitImageRepositories = [];
    }
  }

  onGitActionSelected(event: any) {
    console.log('onGitActionSelected llamado con:', event);
    this.gitAction = event.value;
    this.selectedExistingImage = null;
    this.gitImageName = '';
    
    // Si se selecciona 'update' y ya hay un repositorio Git seleccionado, cargar los repositorios de imágenes
    if (event.value === 'update' && this.selectedGitRepository) {
      this.loadGitImageRepositories(this.selectedGitRepository);
    }
    
    console.log('Antes del setTimeout');
    // Hacer scroll hacia la sección de partición después de un delay más largo
    setTimeout(() => {
      console.log('Dentro del setTimeout, llamando a scrollToPartitionSection');
      this.scrollToPartitionSection();
    }, 300);
  }

  onMonolithicActionSelected(event: any) {
    console.log('onMonolithicActionSelected llamado con:', event);
    this.monolithicAction = event.value;
    this.selectedImage = null;
    this.name = '';
    
    // Si se selecciona 'update', cargar las imágenes existentes
    if (event.value === 'update') {
      this.loadImages();
    }
    
    console.log('Antes del setTimeout (monolithic)');
    // Hacer scroll hacia la sección de partición después de un delay más largo
    setTimeout(() => {
      console.log('Dentro del setTimeout (monolithic), llamando a scrollToPartitionSection');
      this.scrollToPartitionSection();
    }, 300);
  }

  loadExistingImages() {
    if (!this.selectedExistingImage) return;
    
    this.loadingExistingImages = true;
    // Aquí deberías hacer el GET al endpoint externo
    // Por ahora uso un endpoint de ejemplo, ajusta según tu API
    const url = `${this.baseUrl}/images?gitImageRepository.id=${this.selectedExistingImage.id}&page=1&itemsPerPage=100`;
    
    this.http.get(url).subscribe(
      (response: any) => {
        this.existingImages = response['hydra:member'] || [];
        this.loadingExistingImages = false;
      },
      (error) => {
        console.error('Error al cargar las imágenes existentes:', error);
        this.loadingExistingImages = false;
        this.toastService.error('Error al cargar las imágenes existentes');
      }
    );
  }

  resetGitSelections() {
    this.selectedGitRepository = null;
    this.selectedExistingImage = null;
    this.gitImageName = '';
    this.gitAction = 'create';
    this.existingImages = [];
    this.gitRepositories = [];
    this.gitImageRepositories = [];
    
    this.selectedImage = null;
    this.name = '';
    this.monolithicAction = 'create';
  }

  resetCanonicalName() {
    this.name = this.selectedImage ? this.selectedImage.name : '';
  }

  save(): void {
    if (!this.selectedPartition) {
      this.toastService.error('Debes seleccionar una partición');
      return;
    }

    if (this.imageType === 'git') {
      if (!this.selectedGitRepository) {
        this.toastService.error('Debes seleccionar un repositorio Git');
        return;
      }
      if (this.gitAction === 'update' && !this.selectedExistingImage) {
        this.toastService.error('Debes seleccionar un repositorio de imágenes Git');
        return;
      }
    }

    if (this.imageType === 'monolithic') {
      if (this.monolithicAction === 'create' && !this.name) {
        this.toastService.error('Debes introducir un nombre canónico para la imagen');
        return;
      }
      if (this.monolithicAction === 'update' && !this.selectedImage) {
        this.toastService.error('Debes seleccionar una imagen para actualizar');
        return;
      }
    }

    if (this.selectedImage) {
      this.toastService.warning('Aviso: Está seleccionando una imagen previamente creada. Se procede a crear un backup de la misma. ');
    }

    const dialogRef = this.dialog.open(QueueConfirmationModalComponent, {
      width: '400px',
      disableClose: true,
      hasBackdrop: true,
      backdropClass: 'non-clickable-backdrop'
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result !== undefined) {
        this.loading = true;
        
        let payload: any = {
          client: `/clients/${this.clientId}`,
          partition: this.selectedPartition['@id'],
          source: 'assistant',
          type: this.imageType,
          queue: result
        };

        if (this.imageType === 'git') {
          payload.gitRepository = this.selectedGitRepository.name
          payload.name = this.selectedGitRepository.name;
          
          if (this.gitAction === 'create') {
            payload.action = 'create';
          } else {
            payload.action = 'update';
          }
        } else {
          if (this.monolithicAction === 'create') {
            payload.name = this.name;
            payload.action = 'create';
          } else {
            payload.selectedImage = this.selectedImage['@id'];
            payload.action = 'update';
          }
        }

        this.http.post(`${this.baseUrl}/images`, payload)
          .subscribe({
            next: (response) => {
              let actionText = 'creación';
              if (this.imageType === 'git' && this.gitAction === 'update') {
                actionText = 'actualización';
              } else if (this.imageType === 'monolithic' && this.monolithicAction === 'update') {
                actionText = 'actualización';
              }
              this.toastService.success(`Petición de ${actionText} de imagen enviada`);
              this.loading = false;
              this.router.navigate(['/commands-logs']);
            },
            error: (error) => {
              this.toastService.error(error.error['hydra:description']);
              this.loading = false;
            }
          });
      }
    });
  }

  openCreateRepositoryModal(): void {
    this.creatingRepository = true;
    const dialogRef = this.dialog.open(CreateRepositoryModalComponent, {
      width: '600px',
      disableClose: true,
      hasBackdrop: true,
      backdropClass: 'non-clickable-backdrop',
      data: {
        clientRepository: this.selectedRepository
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      this.creatingRepository = false;
      if (result) {
        this.loadGitRepositories();
        setTimeout(() => {
          const newRepository = this.gitRepositories.find(repo => repo['@id'] === result['@id']);
          if (newRepository) {
            this.selectedGitRepository = newRepository;
            this.onGitRepositorySelected(newRepository);
          }
        }, 200);
      }
    });
  }

  get imageType(): string {
    return this._imageType;
  }

  set imageType(value: string) {
    this._imageType = value;
    this.loadImages();
    if (value === 'git') {
      this.loadGitRepositories();
      this.selectedImage = null;
      this.name = '';
      this.monolithicAction = 'create';
    } else {
      this.resetGitSelections();
    }
  }

  onGitImageRepositorySelected(gitImageRepository: any) {
    this.selectedExistingImage = gitImageRepository;
    this.existingImages = [];
    
    if (gitImageRepository) {
      this.loadExistingImages();
    }
  }

  scrollToPartitionSection() {
    const partitionSection = document.getElementById('partition-selection');
    
    if (partitionSection) {
      partitionSection.scrollIntoView({ 
        behavior: 'smooth', 
        block: 'start' 
      });
    }
  }

  get selectedPartition(): any {
    return this._selectedPartition;
  }

  set selectedPartition(value: any) {
    this._selectedPartition = value;
  }
}
