import {Component, EventEmitter, OnInit, Output} from '@angular/core';
import { MatTableDataSource } from "@angular/material/table";
import { SelectionModel } from "@angular/cdk/collections";
import { HttpClient } from "@angular/common/http";
import { ToastrService } from "ngx-toastr";
import { ActivatedRoute, Router } from "@angular/router";
import { ConfigService } from '@services/config.service';
import {CreateTaskComponent} from "../../../../commands/commands-task/create-task/create-task.component";
import {MatDialog} from "@angular/material/dialog";
import {QueueConfirmationModalComponent} from "../../../../../shared/queue-confirmation-modal/queue-confirmation-modal.component";

@Component({
  selector: 'app-deploy-image',
  templateUrl: './deploy-image.component.html',
  styleUrl: './deploy-image.component.css'
})
export class DeployImageComponent implements OnInit{
  baseUrl: string;
  @Output() dataChange = new EventEmitter<any>();

  errorMessage = '';
  clientId: string | null = null;
  partitions: any[] = [];
  images: any[] = [];
  selectedImage: any = null;
  selectedMethod: string | null = null;
  private _selectedPartition: any = null;
  mcastIp: string = '';
  mcastPort: Number = 0;
  mcastMode: string = '';
  mcastSpeed: Number = 0;
  mcastMaxClients: Number = 0;
  mcastMaxTime: Number = 0;
  p2pMode: string = '';
  p2pTime: Number = 0;
  client: any = null;
  clientData: any = [];
  loading: boolean = false;
  allSelected = true;
  runScriptContext: any = null;
  ogInstructions: string = '';
  deployImage: boolean = true;
  showInstructions: boolean = false;
  loadingCommits: boolean = false;
  selectedGitRepository: string = '';

  protected p2pModeOptions = [
    { name: 'Leecher', value: 'leecher' },
    { name: 'Peer', value: 'peer' },
    { name: 'Seeder', value: 'seeder' },
  ];
  protected multicastModeOptions = [
    { name: 'Half duplex', value: "half" },
    { name: 'Full duplex', value: "full" },
  ];

  selectedClients: any[] = [];
  selectedModelClient: any = null;
  filteredPartitions: any[] = [];
  selectedRepository: any = null;
  imageType: string = 'monolithic';

  allMethods = [
    { name: 'Multicast', value: 'udpcast' },
    { name: 'Unicast', value: 'unicast' },
    { name: 'Multicast (direct)', value: 'udpcast-direct' },
    { name: 'Unicast (direct)', value: 'unicast-direct' },
    { name: 'Torrent', value: 'p2p' },
  ];

  dataSource = new MatTableDataSource<any>();
  columns = [
    {
      columnDef: 'diskNumber',
      header: 'Disco',
      cell: (partition: any) => `${partition.diskNumber}`
    },
    {
      columnDef: 'partitionNumber',
      header: 'Particion',
      cell: (partition: any) => `${partition.partitionNumber}`
    },
    {
      columnDef: 'size',
      header: 'Tamaño',
      cell: (partition: any) => `${partition.size} MB`
    },
    {
      columnDef: 'filesystem',
      header: 'Sistema de ficheros',
      cell: (partition: any) => `${partition.filesystem}`
    },
    {
      columnDef: 'operativeSystem',
      header: 'SO',
      cell: (partition: any) => `${partition.operativeSystem?.name}`
    }
  ];

  displayedColumns = ['select', ...this.columns.map(column => column.columnDef)];
  selection = new SelectionModel(true, []);

  repositories: string[] = [];
  loadingRepositories: boolean = false;
  branches: string[] = [];
  selectedBranch: string = '';
  loadingBranches: boolean = false;
  commits: any[] = [];
  selectedCommit: any = null;
  private initialGitLoad = true;

  constructor(
    private http: HttpClient,
    private toastService: ToastrService,
    private configService: ConfigService,
    private router: Router,
    private route: ActivatedRoute,
    private dialog: MatDialog,
  ) {
    this.baseUrl = this.configService.apiUrl;
    this.route.queryParams.subscribe(params => {
      if (params['clientData']) {
        this.clientData = JSON.parse(params['clientData']);
      }
      if (params['runScriptContext']) {
        this.runScriptContext = params['runScriptContext'];
      }
    });
    this.clientId = this.clientData?.length ? this.clientData[0]['@id'] : null;
    this.clientData.forEach((client: { selected: boolean; status: string}) => { client.selected = true; });

    this.selectedClients = this.clientData.filter((client: { selected: boolean; status: string}) => client.selected);

    this.selectedModelClient = this.clientData.find((client: { selected: boolean; status: string}) => client.selected) || null;

    if (this.selectedModelClient) {
      this.loadPartitions(this.selectedModelClient);
    }
  }

  ngOnInit(): void {
    this.route.queryParams.subscribe(params => {
      this.runScriptContext = params['runScriptContext'] ? JSON.parse(params['runScriptContext']) : null;
    });
  }

  onImageTypeSelected(event: any) {
    this.imageType = event;
    if (event === 'git') {
      this.selectedMethod = null;
      this.loadGitRepositories();
    }
    this.loadImages();
    setTimeout(() => {
      this.scrollToPartitionSection();
    }, 200);
  }

  get runScriptTitle(): string {
    const ctx = this.runScriptContext;
    if (!ctx) {
      return '';
    }
    if (Array.isArray(ctx)) {
      return ctx.map(c => c.name).join(', ');
    }
    if (typeof ctx === 'object' && 'name' in ctx) {
      return ctx.name;
    }
    return String(ctx);
  }


  isMethod(method: string): boolean {
    return this.selectedMethod === method;
  }

  toggleClientSelection(client: any) {
    client.selected = !client.selected;
    this.updateSelectedClients();
  }

  updateSelectedClients() {
    this.selectedClients = this.clientData.filter(
      (client: { selected: boolean; state: string }) => client.selected && client.state === "og-live"
    );

    if (!this.selectedClients.includes(this.selectedModelClient)) {
      this.selectedModelClient = null;
      this.filteredPartitions = [];
    }
  }

  getPartitionsTooltip(client: any): string {
    if (!client.partitions || client.partitions.length === 0) {
      return 'No hay particiones disponibles';
    }

    return client.partitions
      .map((p: { partitionNumber: any; size: any; filesystem: any }) => `#${p.partitionNumber} ${p.filesystem} - ${p.size / 1024 }GB`)
      .join('\n');
  }

  loadPartitions(client: any) {
    if (client.selected) {
      this.http.get(`${this.baseUrl}${client.uuid}`).subscribe(
        (fullClientData: any) => {
          this.filteredPartitions = fullClientData.partitions;
          this.selectedRepository = fullClientData.repository ?? null;

          if (fullClientData.partitions) {
            this.filteredPartitions = fullClientData.partitions.filter((partition: any) => {
              return partition.partitionNumber !== 0;
            });
            this.p2pMode = fullClientData.organizationalUnit?.networkSettings?.p2pMode;
            this.p2pTime = fullClientData.organizationalUnit?.networkSettings?.p2pTime;
            this.mcastSpeed = fullClientData.organizationalUnit?.networkSettings?.mcastSpeed;
            this.mcastMode = fullClientData.organizationalUnit?.networkSettings?.mcastMode;
            this.mcastPort = fullClientData.organizationalUnit?.networkSettings?.mcastPort;
            this.mcastIp = fullClientData.organizationalUnit?.networkSettings?.mcastIp;
          }

          this.loadImages();
        },
        (error) => {
          console.error('Error al cargar las particiones:', error);
        }
      );
    } else {
      this.selectedClients = this.selectedClients.filter(c => c.uuid !== client.uuid);
      this.filteredPartitions = [];
    }
  }


  toggleSelectAll() {
    this.allSelected = !this.allSelected;
    this.clientData.forEach((client: { selected: boolean; status: string }) => { client.selected = this.allSelected; });
  }

  loadImages() {
    const repositoryId = this.selectedRepository?.id;

    if (!repositoryId) {
      console.error('Error: No se encontró repositoryId en el cliente seleccionado.');
      return;
    }

    let url = ''

    if (this.imageType === 'monolithic') {
      url = `${this.baseUrl}/image-image-repositories?status=success&repository.id=${repositoryId}&page=1&itemsPerPage=1000`;
    } else {
      url = `${this.baseUrl}/git-image-repositories?status=success&repository.id=${repositoryId}&page=1&itemsPerPage=1000`;
    }

    this.http.get(url).subscribe(
      (response: any) => {
        this.images = response?.['hydra:member'] || [];
      },
      (error) => {
        console.error('Error al cargar las imágenes:', error);
      }
    );
  }

  validateImageSize() {
    if (this.selectedImage && this.selectedPartition) {
      if ((this.selectedImage.datasize / 1024) / 1024 > this.selectedPartition.size) {

        this.errorMessage = "El tamaño de la imagen seleccionada excede el tamaño de la partición.";
        return false;
      }
    }
    this.errorMessage = "";
    
    if (this.selectedMethod) {
      setTimeout(() => {
        this.scrollToPartitionSection();
      }, 300);
    }
    
    return true;
  }

  save(): void {
    if (!this.selectedClients.length) {
      this.toastService.error('Debe seleccionar al menos un cliente');
      return;
    }

    if (!this.selectedImage && this.imageType !== 'git') {
      this.toastService.error('Debe seleccionar una imagen');
      return;
    }

    if (!this.selectedMethod && this.imageType !== 'git') {
      this.toastService.error('Debe seleccionar un método');
      return;
    }

    if (!this.selectedPartition) {
      this.toastService.error('Debe seleccionar una partición');
      return;
    }

    if (this.imageType === 'git' && !this.selectedCommit) {
      this.toastService.error('Debe seleccionar un commit');
      return;
    }

    const dialogRef = this.dialog.open(QueueConfirmationModalComponent, {
      width: '400px',
      disableClose: true,
      hasBackdrop: true,
      backdropClass: 'non-clickable-backdrop'
    });

    this.toastService.info('Preparando petición de despliegue');

    dialogRef.afterClosed().subscribe(result => {
      if (result !== undefined) {
        this.loading = true;
        
        let payload: any;
        let url: string;

        if (this.imageType === 'git') {
          payload = {
            type: 'git',
            clients: this.selectedClients.map((client: any) => client.uuid),
            diskNumber: this.selectedPartition.diskNumber,
            partitionNumber: this.selectedPartition.partitionNumber,
            repositoryName: this.selectedGitRepository,
            branch: this.selectedBranch,
            hexsha: this.selectedCommit.hexsha,
            queue: result
          };
          url = `${this.baseUrl}/git-repositories/deploy-image`;
        } else {
          payload = {
            clients: this.selectedClients.map((client: any) => client.uuid),
            method: this.selectedMethod,
            diskNumber: this.selectedPartition.diskNumber,
            partitionNumber: this.selectedPartition.partitionNumber,
            p2pMode: this.p2pMode,
            p2pTime: this.p2pMode === 'seeder' ? this.p2pTime : null,
            mcastIp: this.mcastIp,
            mcastPort: this.mcastPort,
            mcastMode: this.mcastMode,
            mcastSpeed: this.mcastSpeed,
            maxTime: this.mcastMaxTime,
            maxClients: this.mcastMaxClients,
            type: this.imageType,
            queue: result
          };
          url = `${this.baseUrl}/image-image-repositories/${this.selectedImage.uuid}/deploy-image`;
        }

        this.http.post(url, payload)
          .subscribe({
            next: (response) => {
              this.toastService.success('Petición de despliegue enviada correctamente');
              this.loading = false;
              this.router.navigate(['/commands-logs']);
            },
            error: (error) => {
              this.toastService.error(error.error['hydra:description'], 'Se ha detectado un error en el despliegue de imágenes.', {
                "closeButton": true,
                "newestOnTop": false,
                "progressBar": false,
                "positionClass": "toast-bottom-right",
                "timeOut": 0,
                "extendedTimeOut": 0,
                "tapToDismiss": false
              });
              this.loading = false;
            }
          });
      }
    });
  }

  isFormValid(): boolean {
    if (!this.allSelected || !this.selectedModelClient || !this.selectedPartition) {
      return false;
    }

    if (this.imageType === 'git') {
      if (!this.selectedCommit) {
        return false;
      }
    }

    if (this.imageType !== 'git' && !this.selectedMethod) {
      return false;
    }

    if (this.imageType !== 'git') {
      if (this.isMethod('udpcast') || this.isMethod('uftp') || this.isMethod('udpcast-direct')) {
        if (!this.mcastPort || !this.mcastIp || !this.mcastMode || !this.mcastSpeed || !this.mcastMaxClients || !this.mcastMaxTime) {
          return false;
        }
      }

      if (this.isMethod('p2p')) {
        if (!this.p2pMode) {
          return false;
        }
        if (this.p2pMode === 'seeder' && !this.p2pTime) {
          return false;
        }
      }
    }

    return true;
  }


  openScheduleModal(): void {
    const dialogRef = this.dialog.open(CreateTaskComponent, {
      width: '800px',
      data: {
        scope: this.runScriptContext.type,
        organizationalUnit: this.runScriptContext['@id'],
        source: 'assistant'
      }
    });

    dialogRef.afterClosed().subscribe((result: { [x: string]: any; }) => {
      if (result) {
        const payload = {
          method: this.selectedMethod,
          diskNumber: this.selectedPartition.diskNumber,
          partitionNumber: this.selectedPartition.partitionNumber,
          p2pMode: this.selectedMethod === 'p2p' ? this.p2pMode : null,
          p2pTime: this.selectedMethod === 'p2p' && this.p2pMode === 'seeder' ? this.p2pTime : null,
          mcastIp: this.selectedMethod === 'udpcast' ? this.mcastIp : null,
          mcastPort: this.selectedMethod === 'udpcast' ? this.mcastPort : null,
          mcastMode: this.selectedMethod === 'udpcast' ? this.mcastMode : null,
          mcastSpeed: this.selectedMethod === 'udpcast' ? this.mcastSpeed : null,
          maxTime: this.selectedMethod === 'udpcast' ? this.mcastMaxTime : null,
          maxClients: this.selectedMethod === 'udpcast' ? this.mcastMaxClients : null,
        };

        this.http.post(`${this.baseUrl}/command-task-scripts`, {
          commandTask: result['taskId'] ? result['taskId']['@id'] : result['@id'],
          parameters: payload,
          order: result['executionOrder'] || 1,
          type: 'deploy-image',
        }).subscribe({
          next: () => {
            this.toastService.success('Script añadido con éxito a la tarea');
          },
          error: (error) => {
            this.toastService.error(error.error['hydra:description']);
          }
        })
      }
    });
  }

  generateOgInstructions() {
    this.showInstructions = true;
    this.ogInstructions = `og-deploy-image --image ${this.selectedImage.name} --partition ${this.selectedPartition.partitionNumber} --method ${this.selectedMethod}`;
  }

  scrollToPartitionSection() {    
    const partitionSection = document.getElementById('partition-selection');    
    if (partitionSection) {
      partitionSection.scrollIntoView({ 
        behavior: 'smooth', 
        block: 'start' 
      });
      console.log('Scroll ejecutado');
    }
  }

  scrollToAdvancedOptions() {    
    const advancedOptions = document.getElementById('advanced-options');
    
    if (advancedOptions) {
      advancedOptions.scrollIntoView({ 
        behavior: 'smooth', 
        block: 'start' 
      });
      console.log('Scroll hacia opciones avanzadas ejecutado');
    }
  }

  get selectedPartition(): any {
    return this._selectedPartition;
  }

  set selectedPartition(value: any) {
    this._selectedPartition = value;
  }

  loadGitRepositories() {
    this.loadingRepositories = true;
    this.http.get<any>(`${this.baseUrl}/image-repositories/server/git/${this.selectedRepository?.uuid}/get-collection`).subscribe(
      data => {
        this.repositories = data.repositories || [];
        this.loadingRepositories = false;
        if (this.repositories.length > 0) {
          this.selectedGitRepository = this.repositories[0];
          this.loadGitBranches();
        }
      },
      error => {
        this.toastService.error('Error al cargar los repositorios git');
        this.loadingRepositories = false;
      }
    );
  }

  onGitRepositoryChange(event: any) {
    this.selectedGitRepository = event;
    this.selectedBranch = '';
    this.branches = [];
    this.commits = [];
    this.selectedCommit = null;
    this.loadGitBranches();
  }

  loadGitBranches() {
    if (!this.selectedGitRepository) return;
    this.loadingBranches = true;
    this.http.post<any>(`${this.baseUrl}/image-repositories/server/git/${this.selectedRepository?.uuid}/branches`, { repositoryName: this.selectedGitRepository }).subscribe(
      data => {
        this.branches = data.branches || [];
        this.loadingBranches = false;
        if (this.branches.length > 0) {
          this.selectedBranch = this.branches[0];
          this.loadGitCommits();
        }
      },
      error => {
        this.toastService.error('Error al cargar las ramas');
        this.loadingBranches = false;
      }
    );
  }

  onGitBranchChange() {
    this.selectedCommit = null;
    this.commits = [];
    this.loadGitCommits();
  }

  loadGitCommits() {
    if (!this.selectedGitRepository || !this.selectedBranch) return;
    this.loadingCommits = true;
    this.http.post<any>(`${this.baseUrl}/image-repositories/server/git/${this.selectedRepository?.uuid}/commits`, {
      repositoryName: this.selectedGitRepository,
      branch: this.selectedBranch
    }).subscribe(
      data => {
        this.commits = data.commits || [];
        this.loadingCommits = false;
      },
      error => {
        this.toastService.error('Error al cargar los commits');
        this.loadingCommits = false;
      }
    );
  }
}
