import { Component, OnInit, OnDestroy, Input } from '@angular/core';

@Component({
  selector: 'app-scroll-to-top',
  templateUrl: './scroll-to-top.component.html',
  styleUrls: ['./scroll-to-top.component.css']
})
export class ScrollToTopComponent implements OnInit, OnDestroy {
  @Input() threshold: number = 300; 
  @Input() targetElement: string = '.header-container'; 
  @Input() position: 'bottom-right' | 'bottom-left' | 'top-right' | 'top-left' = 'bottom-right';
  @Input() showTooltip: boolean = true;
  @Input() tooltipText: string = 'Volver arriba';
  @Input() tooltipPosition: 'left' | 'right' | 'above' | 'below' = 'left';

  showScrollToTop: boolean = false;
  private scrollListener: (() => void) | undefined;

  ngOnInit(): void {
    this.setupScrollListener();
  }

  ngOnDestroy(): void {
    if (this.scrollListener) {
      window.removeEventListener('scroll', this.scrollListener);
    }
  }

  private setupScrollListener(): void {
    this.scrollListener = () => {
      this.showScrollToTop = window.scrollY > this.threshold;
    };
    window.addEventListener('scroll', this.scrollListener);
    
    this.scrollListener();
  }

  scrollToTop(): void {
    try {
      const targetElement = document.querySelector(this.targetElement);
      if (targetElement) {
        targetElement.scrollIntoView({
          behavior: 'smooth',
          block: 'start'
        });
      } else {
        window.scrollTo({
          top: 0,
          left: 0,
          behavior: 'smooth'
        });
      }
    } catch (error) {
      window.scrollTo(0, 0);
    }
  }

  getPositionClass(): string {
    return `scroll-to-top-button-${this.position}`;
  }
} 